package envoy.client.ui;

import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.IOException;
import java.util.Properties;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

import envoy.client.Client;
import envoy.client.Config;
import envoy.schema.Message;
import envoy.schema.User;
import envoy.schema.Users;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>ChatWindow.java</strong><br>
 * Created: <strong>28 Sep 2019</strong><br>
 * 
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @since Envoy 0.1
 */
public class ChatWindow extends JFrame {

	private static final long serialVersionUID = 6865098428255463649L;

	private long	recipientID	= 0;
	private JPanel	contentPane	= new JPanel();

	private static Client client;

	private DefaultListModel<String> messageListModel = new DefaultListModel<>();

	public ChatWindow() {
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 600, 800);
		setTitle("Envoy");
		setLocationRelativeTo(null);

		contentPane.setBackground(new Color(0, 0, 0));
		contentPane.setForeground(Color.white);
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		GridBagLayout gbl_contentPane = new GridBagLayout();
		gbl_contentPane.columnWidths	= new int[] { 1, 1, 1 };
		gbl_contentPane.rowHeights		= new int[] { 1, 1, 1 };
		gbl_contentPane.columnWeights	= new double[] { 0.3, 1.0, 0.1 };
		gbl_contentPane.rowWeights		= new double[] { 0.05, 1, 0.07 };
		contentPane.setLayout(gbl_contentPane);

		JList<String> elementList = new JList<>();
		elementList.setFocusTraversalKeysEnabled(false);

		elementList.setSelectionForeground(new Color(255, 255, 255));
		elementList.setSelectionBackground(new Color(102, 0, 153));
		elementList.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		elementList.setForeground(new Color(255, 255, 255));
		elementList.setBackground(new Color(51, 51, 51));

		elementList.setModel(messageListModel);
		elementList.setFont(new Font("Arial", Font.PLAIN, 17));
		elementList.setFixedCellHeight(60);
		elementList.setBorder(new EmptyBorder(5, 5, 5, 5));

		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setForeground(new Color(0, 0, 0));
		scrollPane.setBackground(new Color(51, 51, 51));
		scrollPane.setViewportView(elementList);
		scrollPane.setBorder(null);

		GridBagConstraints gbc_scrollPane = new GridBagConstraints();
		gbc_scrollPane.fill			= GridBagConstraints.BOTH;
		gbc_scrollPane.gridwidth	= 2;
		gbc_scrollPane.gridx		= 1;
		gbc_scrollPane.gridy		= 1;

		gbc_scrollPane.insets = new Insets(10, 10, 10, 10);

		contentPane.add(scrollPane, gbc_scrollPane);

		// Message enter field
		JTextArea messageEnterTextfield = new JTextArea();
		messageEnterTextfield.setCaretColor(new Color(255, 255, 255));
		messageEnterTextfield.setForeground(new Color(255, 255, 255));
		messageEnterTextfield.setBackground(new Color(51, 51, 51));
		messageEnterTextfield.setLineWrap(true);
		messageEnterTextfield.setBorder(null);
		messageEnterTextfield.setFont(new Font("Arial", Font.PLAIN, 17));
		messageEnterTextfield.setBorder(new EmptyBorder(5, 5, 5, 5));

		GridBagConstraints gbc_moveSelectionMessageEnterTextfield = new GridBagConstraints();
		gbc_moveSelectionMessageEnterTextfield.fill		= GridBagConstraints.BOTH;
		gbc_moveSelectionMessageEnterTextfield.gridx	= 1;
		gbc_moveSelectionMessageEnterTextfield.gridy	= 2;

		gbc_moveSelectionMessageEnterTextfield.insets = new Insets(10, 10, 10, 10);

		contentPane.add(messageEnterTextfield, gbc_moveSelectionMessageEnterTextfield);

		// Post Button
		JButton postButton = new JButton("Post");
		postButton.setForeground(new Color(255, 255, 255));
		postButton.setBackground(new Color(102, 51, 153));
		postButton.setBorderPainted(false);

		GridBagConstraints gbc_moveSelectionPostButton = new GridBagConstraints();

		gbc_moveSelectionPostButton.fill	= GridBagConstraints.BOTH;
		gbc_moveSelectionPostButton.gridx	= 2;
		gbc_moveSelectionPostButton.gridy	= 2;

		gbc_moveSelectionPostButton.insets = new Insets(10, 10, 10, 10);

		contentPane.add(postButton, gbc_moveSelectionPostButton);

		postButton.addActionListener((evt) -> {
			if (recipientID == 0) System.out.println("Please select recipient");

			// TODO: Acquire proper sender id
			if (!messageEnterTextfield.getText().isEmpty() && recipientID != 0) try {
				final Message message = client.createMessage(1, recipientID, messageEnterTextfield.getText());
				client.sendMessage(message);
				appendMessageToChat(message);
				messageEnterTextfield.setText("");
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this,
						"An exception occured while sending a message. See the log for more details.",
						"Exception occured",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
		});

		JList<User> userList = new JList<>();
		userList.setCellRenderer(new UserListRenderer());
		userList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		userList.addListSelectionListener((listSelectionEvent) -> {
			if (!listSelectionEvent.getValueIsAdjusting()) {
				@SuppressWarnings("unchecked")
				JList<User> selectedUserList = (JList<User>) listSelectionEvent.getSource();
				recipientID = selectedUserList.getModel().getElementAt(selectedUserList.getSelectedIndex()).getID();
			}
		});

		userList.setSelectionForeground(new Color(255, 255, 255));
		userList.setSelectionBackground(new Color(102, 0, 153));
		userList.setForeground(new Color(255, 255, 255));
		userList.setBackground(new Color(51, 51, 51));
		userList.setFont(new Font("Arial", Font.PLAIN, 17));
		userList.setBorder(new EmptyBorder(5, 5, 5, 5));

		GridBagConstraints gbc_userList = new GridBagConstraints();
		gbc_userList.gridx	= 0;
		gbc_userList.gridy	= 1;
		gbc_userList.anchor	= GridBagConstraints.PAGE_START;
		gbc_userList.insets	= new Insets(10, 0, 10, 0);

		contentPane.add(userList, gbc_userList);
		contentPane.revalidate();

		loadUserList(userList);
	}

	/**
	 * Initializes the elements of the user list by downloading them from the
	 * server.
	 * 
	 * @param userList The {@link JList} to put the elements in
	 */
	private void loadUserList(JList<User> userList) {
		new Thread(() -> {
			Users					users			= client.getUsersListXml();
			DefaultListModel<User>	userListModel	= new DefaultListModel<>();
			users.getUser().forEach(user -> userListModel.addElement(user));
			SwingUtilities.invokeLater(() -> userList.setModel(userListModel));
		}).start();
	}

	/**
	 * Extracts the first text content from a message.
	 * 
	 * @param message The message from which to return the text content
	 * @return The first content of type 'text'
	 */
	private String getFirstTextContent(Message message) { return message.getContent().get(0).getText(); }

	/**
	 * Appends a message with sender and message content to the message list.
	 * 
	 * @param message The message to append
	 */
	private void appendMessageToChat(Message message) {
		messageListModel.addElement("<html>" + "<p style=\"color:#d2d235\"> <b> <small>"
				+ message.getMetaData().getSender() + "</b> </small>" + "<br>" + "<p style=\"color:white\">"
				+ getFirstTextContent(message) + "</span></html>");
	}

	public static void main(String[] args) {
		Config config = new Config();
		if (args.length > 0) {
			config.load(args);
		} else {
			ClassLoader loader = Thread.currentThread().getContextClassLoader();
			try {
				Properties configProperties = new Properties();
				configProperties.load(loader.getResourceAsStream("server.properties"));
				config.load(configProperties);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		if(!config.isInitialized()) {
			System.err.println("Server or port are not defined. Exiting...");
			System.exit(1);
		}
		
		client = new Client(config);

		EventQueue.invokeLater(() -> {
			try {
				ChatWindow frame = new ChatWindow();
				frame.setVisible(true);
			} catch (Exception e) {
				e.printStackTrace();
			}
		});
	}
}